import sys
import os
import urllib.parse
from requests import get
import xmltodict
from datetime import datetime, timedelta, date
import json

# -------------------
# Imports for variable file
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))

from variables.pubmed import *
# -------------------

CATEGORIES = {
    "Access" : ["Availability", "Affordability", "Essential medecins", "Care therapy ", "Care health", "Health Expenditures", "Health care costs", "Market", "Special populations", "Child Health", "Womens Health", "Age", "Minority", "Primary Care", "Specialty Care", "Patient acceptance", "Patient centered care", "Prevention and control", "Mass screening", "Palliative care", "Quality", "Telemedicine", "Digital health", "Supplies", "Human Resources", "Enablers/barriers", "Gender equity", "Racial", "Equity"],
    "Care/Management" : ["Clinical", "Health promotion", "Health education", "Research & Innovation", "Therapeutic Development", "Technological Development", "Self-management", "Self-monitoring", "Dosing", "Injections", "Primary Care", "Secondary Care", "Integrated Care", "Treatment management", "Immunization", "Vaccination", "Prevention and control", "Mass screening", "Palliative care", "Adherence", "Control", "Rehabilitation services", "Clinical guidelines"],
    "Policy" : ["Health policy", "Healthcare policy", "National health policy", "Regional health policy", "Health legislation", "Policy evaluation", "Policy analysis", "Policy formulation", "Regulation", "Governance", "Global initiatives and organizations ", "Universal Health Care", "Expansion", "Health insurance", "Coverage", "Funding and investment", "Health planning", "Health reform", "Policy monitoring"],
    "Advocacy" : ["Public health campaign", "Policy lobbying", "Patient advocacy", "Justice", "Awareness campaign", "Education", "Corporate accountability", "Social determinants of health", "Empowerment", "Community", "Peer support", "Civil society"],
    "Education" : ["Patient education", "Parent education", "Educational materials", "Community heatlh education", "Awareness ", "Community engagement", "Health literacy", "Medical education", "Training program", "Technology education", "Medical devices", "Information Dissemination", "Digital health", "Behavioral change", "Nutrition education", "Risk communication", "Sector integration"]
}

KEYWORDS_GROUPS = {
    "exact match" : KEYWORDS_MESH_TERM,
    "site proposition" : KEYWORDS_MESH_SITE_PROPOSITION,
    "personal proposition" : KEYWORDS_MESH_PROPOSITION,
    "subheading" : KEYWORDS_MESH_SUBHEADING
}

def getPubmedData(term, date_min, date_max):

    print("--------------------")
    print(f"Term: {term}")
    print("--------------------")
    print("--------------------")
    print(f"Date min: {date_min}")
    print(f"Date max: {date_max}")
    print("--------------------")

    # Search : 
    #   db: pubmed
    #   term : search term
    #   retmode : return data in json format
    #   mindate : start date
    #   maxdate : end date
    #   usehistory : 
    url = f'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&api_key={PUBMED_API_KEY}&term={term}&retmode=json&mindate={date_min}&maxdate={date_max}&usehistory=y'

    while(True):
        try:  
            response = get(url)
            break
        except Exception as e:
            print(e)

    search_res = response.json()

    query_key = search_res["esearchresult"]["querykey"]
    webenv = search_res["esearchresult"]["webenv"]

    print(f"Count : {search_res["esearchresult"]["count"]}")

    print("--------------------")
    print(f"Query key: {query_key}")
    print(f"Web env: {webenv}")
    print("--------------------")

    # Fetch article data from articles PMIDs that we got from the search
    url = f"https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&query_key={query_key}&WebEnv={webenv}"

    while(True):
        try:
            response = get(url)
            break
        except Exception as e:
            print(e)

    obj = xmltodict.parse(response.text)
    if "PubmedArticleSet" not in obj:
        return []
        
    obj = obj["PubmedArticleSet"]

    data_list = []

    for key in obj:
        if isinstance(obj[key], list):
            for entrie in obj[key]:
                if "MedlineCitation" in entrie:

                    if "MeshHeadingList" in entrie["MedlineCitation"]:
                        data = {}

                        # PMID
                        data["PMID"] = entrie["MedlineCitation"]["PMID"]["#text"]

                        # Journal title
                        data["Title"] = ""
                        if isinstance(entrie["MedlineCitation"]["Article"]["Journal"]["Title"], list):
                            for part in entrie["MedlineCitation"]["Article"]["Journal"]["Title"]:
                                if "#text" in part:
                                    data["Title"] += part["#text"]
                        elif "#text" in entrie["MedlineCitation"]["Article"]["Journal"]["Title"] and not isinstance(entrie["MedlineCitation"]["Article"]["Journal"]["Title"], str):
                            data["Title"] = entrie["MedlineCitation"]["Article"]["Journal"]["Title"]["#text"]
                        else:
                            data["Title"] = entrie["MedlineCitation"]["Article"]["Journal"]["Title"]
                        
                        # Article title
                        data["ArticleTitle"] = ""
                        if isinstance(entrie["MedlineCitation"]["Article"]["ArticleTitle"], list):
                            for part in entrie["MedlineCitation"]["Article"]["ArticleTitle"]:
                                if "#text" in part:
                                    data["ArticleTitle"] += part["#text"]
                        elif "#text" in entrie["MedlineCitation"]["Article"]["ArticleTitle"] and not isinstance(entrie["MedlineCitation"]["Article"]["ArticleTitle"], str):
                            data["ArticleTitle"] = entrie["MedlineCitation"]["Article"]["ArticleTitle"]["#text"]
                        else:
                            data["ArticleTitle"] = entrie["MedlineCitation"]["Article"]["ArticleTitle"]
                        
                        # Abstarct
                        data["Abstract"] = ""
                        if "Abstract" in entrie["MedlineCitation"]["Article"] :
                            if isinstance(entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"], list):
                                for part in entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"]:
                                    if "#text" in part:
                                        data["Abstract"] += part["#text"]
                            elif "#text" in entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"] and not isinstance(entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"], str):
                                data["Abstract"] = entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"]["#text"]
                            else:
                                data["Abstract"] = entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"]

                        # MeSH terms
                        data["MeshTerms"] = []
                        if isinstance(entrie["MedlineCitation"]["MeshHeadingList"]["MeshHeading"], list):
                            for meshTerm in entrie["MedlineCitation"]["MeshHeadingList"]["MeshHeading"]:
                                data["MeshTerms"].append(meshTerm["DescriptorName"]["#text"])
                        else:
                            data["MeshTerms"].append(entrie["MedlineCitation"]["MeshHeadingList"]["MeshHeading"]["DescriptorName"]["#text"])

                        # PubMed publication date
                        for date in entrie["PubmedData"]["History"]["PubMedPubDate"]:
                            if date["@PubStatus"] == "pubmed":
                                data["Date"] = {
                                    "Year": date["Year"],
                                    "Month": date["Month"],
                                    "Day": date["Day"]
                                }
                                break

                        data_list.append(data)
    
        return data_list

def url_encode(text):
    return urllib.parse.quote_plus(text, safe='[]():"')

def get_mesh_terms(terms):
    res = []

    for item in terms.values():
        if isinstance(item, list):
            tmp = [f'"{i}"[Mesh:noexp]' for i in item]
            res.append("( " + " AND ".join(tmp) + " )")
        else:
            res.append(f'"{item}"[Mesh:noexp]')
    
    return res

def get_subheadings(terms):
    res = []

    for item in terms.values():
        if isinstance(item, list):
            tmp = [f'"{i}"[SubHeading:noexp]' for i in item]
            res.append("( " + " AND ".join(tmp) + " )")
        else:
            res.append(f'"{item}"[SubHeading:noexp]')
    
    return res

def main():

    SAVE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./tmp"))

    ncds_mesh_terms = get_mesh_terms(NCDS_MESH_TERM)
    keywords_mesh_terms = get_mesh_terms(KEYWORDS_MESH_TERM)
    keywords_site_proposition_mesh_terms = get_mesh_terms(KEYWORDS_MESH_SITE_PROPOSITION)
    keywords_proposition_mesh_terms = get_mesh_terms(KEYWORDS_MESH_PROPOSITION)
    keywords_subheadings = get_subheadings(KEYWORDS_MESH_SUBHEADING)

    keywords_groups = {
        "exact match" : keywords_mesh_terms,
        "site proposition" : keywords_site_proposition_mesh_terms,
        "personal proposition" : keywords_proposition_mesh_terms,
        "subheading" : keywords_subheadings
    }

    stored_articles = []
    stored_pmids = []

    for ncd_mesh_term in ncds_mesh_terms:
        print(f"NCD : {ncd_mesh_term}")

        for keywords in keywords_groups.values():
            search_term = f"{ncd_mesh_term} AND ( {" OR ".join(keywords)} )"

            print(f"Search term : {search_term}")

            # Article PubMed publication date
            end_date = date.today()
            start_date = end_date - timedelta(days=1)

            articles = getPubmedData(url_encode(search_term), start_date, end_date)

            for article in articles:

                if article["PMID"] in stored_pmids:
                    continue

                stored_pmids.append(article["PMID"])

                article_mesh_terms = [mesh.lower() for mesh in article["MeshTerms"]]

                article["NCDs"] = []
                article["Categories"] = []

                # Get NCDs
                for ncd, ncd_mesh in NCDS_MESH_TERM.items():
                    if ncd_mesh.lower() in article_mesh_terms:
                        article["NCDs"].append(ncd)

                # Get categories
                for group in KEYWORDS_GROUPS.values():
                    for keyword, mesh in group.items():
                        present = True
                        if isinstance(mesh, list):
                            for tmp in mesh:
                                if tmp.lower() not in article_mesh_terms:
                                    present = False
                        elif mesh.lower() not in article_mesh_terms:
                            present = False

                        if present:
                            for category in CATEGORIES.keys():
                                if keyword in CATEGORIES[category]:
                                    if category not in article["Categories"]:
                                        article["Categories"].append(category)

                stored_articles.append(article)

    print(stored_articles)

    # store article in json file
    # with open(f"{SAVE_DIR}/articles.json", "w") as json_file:
    #     json.dump(stored_articles, json_file, indent=4)

if __name__ == "__main__":
    main()