# Access Pubmed data info

## Intro 
To access the pubmed database we need to use the Entrez Programming Utilities (E-utilities) of the National Center for Biotechnology Information (NCBI). There's a set of eight server-side programs that provide a stable interface into the Entrez query and database system. All E-utilities calls share the same base URL:
 - https://eutils.ncbi.nlm.nih.gov/entrez/eutils/

## Usage Info
The NCBI recommends  that users post no more than 3 URL requests per second and limit large jobs to either weekends or between 9:00 PM and 5:00 AM Eastern time during weekdays. **Failure to comply with this policy may result in an IP address being blocked from accessing NCBI.**

Source : (Usage Guidelines and Requirements) https://www.ncbi.nlm.nih.gov/books/NBK25497/

## E-utilities

### EInfo
eutils.ncbi.nlm.nih.gov/entrez/eutils/einfo.fcgi

Provides the number of records indexed in each field of a given database, the date of the last update of the database, and the available links from the database to other Entrez databases.

### ESearch
eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi

Responds to a text query with the list of matching UIDs in a given database (for later use in ESummary, EFetch or ELink), along with the term translations of the query.

#### Required Parameters

 - db - database to search
 - term - text query in Entrez format

#### Optional Parameters

 - usehistory - store result on history server
 - WebEnv
 - query_key
 - retstart - index from which it will start retrieving data (default = 0)
 - retmax - how much values are going to be retrieved (default = 20). Documentation says max 10000 ids can be retreived but i didn't have that limitation. I was able to retrieve more than 100000 ids from one request. For the Pubmed database you are able to retreive only the first 10000.
 - rettype 
 - retmode - return format (xml - default or json)
 - sort
 - field
 - idtype
 - datetype
 - reldate
 - mindate, maxdate

### EPost
eutils.ncbi.nlm.nih.gov/entrez/eutils/epost.fcgi

Accepts a list of UIDs from a given database, stores the set on the History Server, and responds with a query key and web environment for the uploaded dataset.

#### Required Parameters
 - db - database containing the UIDS
 - id - UID list than will be added to a history server

#### Optional Parameter
 - WebEnv - Web Environment that will receive the UID list.


### ESummary
eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi

Responds to a list of UIDs from a given database with the corresponding document summaries.

#### Required Parameter
 - db - database from which to retrieve DocSums
 - id - UID list
 - WebEnv - used with query_key
 - query_key - used with WebEnv

#### Optional Parameters
 - retstart
 - retmax
 - retmode
 - version

### EFetch
eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi

Responds to a list of UIDs in a given database with the corresponding data records in a specified format.

#### Required Parameters
 - db - database from which to retrieve records
 - id - UID list
 - WebEnv - used with query_key
 - query_key - used with WebEnv

#### Optional Parameters
 - retmode
 - rettype
 - retstart
 - retmax
 - strand
 - seq_start
 - seq_stop
 - complexity   

### ELink
eutils.ncbi.nlm.nih.gov/entrez/eutils/elink.fcgi

Responds to a list of UIDs in a given database with either a list of related UIDs (and relevancy scores) in the same database or a list of linked UIDs in another Entrez database; checks for the existence of a specified link from a list of one or more UIDs; creates a hyperlink to the primary LinkOut provider for a specific UID and database, or lists LinkOut URLs and attributes for multiple UIDs.

### EGQuery (pas fonctionel)
eutils.ncbi.nlm.nih.gov/entrez/eutils/egquery.fcgi

Responds to a text query with the number of records matching the query in each Entrez database.

### ESpell
eutils.ncbi.nlm.nih.gov/entrez/eutils/espell.fcgi

Retrieves spelling suggestions for a text query in a given database.

#### Required Parameters

 - db - database to search
 - term - text query in Entrez format

### ECitMatch
eutils.ncbi.nlm.nih.gov/entrez/eutils/ecitmatch.cgi

Retrieves PubMed IDs (PMIDs) corresponding to a set of input citation strings.

#### Required Parameters
 - db - database to search
 - bdata - citation strings

## Syntax 

Source: (Syntax and Initial Parsing of Entrez Queries) https://www.ncbi.nlm.nih.gov/books/NBK25497/

Text search strings entered into the Entrez system are converted into Entrez queries with the following format:

term1[field1] Op term2[field2] Op term3[field3] Op ...

where the terms are search terms, each limited to a particular Entrez field in square brackets, combined using one of three Boolean operators: Op = AND, OR, or NOT. These Boolean operators must be typed in all capital letters.

Example: human[organism] AND topoisomerase[protein name]

When searching for terms, it is important to pay attention to how the search is structured. For example, if you search for **breast cancer** without quotes, the search engine will look for any occurrence of the terms **"breast"**, **"cancer"**, or the exact phrase **"breast cancer"**. To specifically search for the phrase **breast cancer** as a whole, you need to enclose it in quotation marks (" ").

![Search with and without quotes](./img/SearchQuotationExample.png)

### API syntax

When using the API, the search term must be properly URL-encoded. Spaces should be replaced by "+", and all other special characters must be replaced by % followed by their ASCII hexadecimal code, except for a small number of URL-safe characters, such as "()" and "[]".

## Content
PubMed is composed of:

- Primarily the MEDLINE database.
- Older references from the print version of Index Medicus, dating back to 1951 and earlier.
- References to some journals before they were indexed in Index Medicus and MEDLINE, such as Science, BMJ, and Annals of Surgery.
- Very recent entries for articles before they are indexed with Medical Subject Headings (MeSH) and added to MEDLINE.
- A collection of full-text books and other subsets of NLM (National Library of Medicine) records.
- Citations from PMC (PubMed Central).
- The NCBI Bookshelf, which provides access to books and documents in life sciences and healthcare.

## MeSH 

Medical Subject Headings (MeSH) are standardised keywords present in the MeSH database to index articles in the MEDLINE database. The MeSH database is a vocabulary thesaurus developed by the National Library of Medicine (NLM) for indexing articles in PubMed. MeSH terms provide information on the content of an article. Changes in medical terminology in published literature is reflected by MeSH headings which are regularly updated by the NLM. They are assigned by trained indexers based on the article's content and MeSH vocabulary.

### Mesh headings

The primary subject or concept that an article is about. It is the main term used for indexing and organizing literature on a specific topic.

### Mesh subheadings

Further refine the topic with specific aspects, such as therapy, diagnosis, surgery, blood, etc...

### Search syntax

Each MeSH term is organized into categories and subcategories, making them hierarchical.

To search for articles that specifically discuss a MeSH term, you need to specify the term followed by "[MeSH]". For example:

```
diabetes[MeSH]
```

This will automatically match your term to a MeSH term if it exists.

You can also visit the MeSH database at https://www.ncbi.nlm.nih.gov/mesh to search for terms. After selecting a term, you can specify a subheading. The subheading is added after the MeSH term with a forward slash (/):

```
"Diabetes Mellitus/surgery"[MeSH]
```

This searches for the MeSH term "Diabetes Mellitus" with the subheading "surgery".

As mentioned earlier, MeSH terms are hierarchical. When you search for a term, PubMed will also search for all related, narrower terms (children of the term). To limit this automatic expansion, you can add :noexp to your request:

```
"Diabetes Mellitus"[MeSH:noexp]
```

This ensures that PubMed will search for the exact MeSH term "Diabetes Mellitus" and not expand to related terms.

## Keywords

Keywords are words or phrases selected by the authors to describe the main content of an article. They are not standardized, meaning they can vary between authors and articles. These keywords help readers quickly identify the key themes or topics discussed in the publication.

## Fields

In PubMed, fields are specific parts of a database record where different types of information about an article are stored. Fields help organize and retrieve information effectively, as they allow you to target searches to specific sections of an article's metadata (such as the title, author, publication date, or MeSH terms).

Each field has a field tag (enclosed in square brackets, e.g., [TI] for Title) that you can use to specify your search query.

When no field tags are specified, PubMed searches across all fields. However, it also applies Automatic Term Mapping (ATM) to map your terms to MeSH terms, journal names, author names, and other indexed fields, which helps broaden the search intelligently.

![ATM exemple during search](./img/SearchATMExample.png)

### ATM

Each time you enter a search term into PubMed, the system checks for that term in what it calls its "Translation Tables".  These are just a set tools that map terms to other terms.

There are a number of different ways a term might get mapped, but one of the primary ways is to a particular Subject Heading or MeSH Term.  Essentially, the database will try to see if there is an official MeSH Term mapped to the keyword you entered.  If it can find a match, then it will search for that MeSH term in addition to the term you entered.

### List

I don't know if it is all the fields because I think there is more fields

- https://www.ncbi.nlm.nih.gov/books/NBK49540/
- (A verifier) https://wayback.archive-it.org/org-350/20240424200258/https://www.nlm.nih.gov/bsd/mms/medlineelements.html#ab

## Operators

When performing a search in PubMed, you can use Boolean operators to structure your query for more precise results. The available operators are: AND, OR, and NOT. These operators must be written in uppercase, as lowercase versions may not be recognized as operators and could be interpreted as search terms instead.

When using these operators, they are applyed in order. For exemple:
- aa AND bb OR cc <=> (aa AND bb) OR cc
- aa OR bb AND cc <=> (aa OR bb) AND cc

## Source
https://www.ncbi.nlm.nih.gov/home/develop/api/
https://www.ncbi.nlm.nih.gov/books/NBK25501/ (All the books)

### API Source

https://pmc.ncbi.nlm.nih.gov/articles/PMC10220169/#R2
https://www.ncbi.nlm.nih.gov/mesh
https://www.youtube.com/watch?v=uyF8uQY9wys
https://library.bath.ac.uk/pubmed/keyword-search
https://research.library.gsu.edu/c.php?g=115556&p=753156
https://en.wikipedia.org/wiki/PubMed
https://library-guides.ucl.ac.uk/pubmed/simple-search-explained
https://www.youtube.com/watch?v=ZdfAd_xFNDw
https://hslguides.osu.edu/pubmed/automatic-term-mapping