# -----------------------------
# Il calcule les statistiques des articles publiés sur PubMed depuis le fichier sauvegarde (./save/save_3_years.json)
# et les stoques dans un fichier json (./data/article_count_local_save.json).
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

import json
import sys
import os
import statistics
from datetime import datetime, timedelta

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))

from variables.pubmed import NCDS_MESH_TERM, KEYWORDS_MESH_TERM, KEYWORDS_MESH_SUBHEADING, KEYWORDS_MESH_SITE_PROPOSITION, KEYWORDS_MESH_PROPOSITION

def lower_keywords(mesh_terms):
    res = []

    for _, mesh_term in mesh_terms.items():
        if isinstance(mesh_term, list):
            res.append([part.lower() for part in mesh_term])
        else:
            res.append(mesh_term.lower())
    
    return res

def get_date_indices(date, start_date):
    delta_days = (date - start_date).days
    day_index = delta_days

    week_index = (delta_days // 7)

    delta_months = (date.year - start_date.year) * 12 + (date.month - start_date.month)
    month_index = delta_months

    return day_index, week_index, month_index

def add_article(article, category, counts, ncd, article_date):
    start_date = datetime(2022, 1, 1)
    day_index, week_index, month_index = get_date_indices(article_date, start_date)

    if article["PMID"] not in counts[ncd][category]["day"][day_index]:
        counts[ncd][category]["day"][day_index].append(article["PMID"])

    if article["PMID"] not in counts[ncd][category]["week"][week_index]:
        counts[ncd][category]["week"][week_index].append(article["PMID"])

    if article["PMID"] not in counts[ncd][category]["month"][month_index]:
        counts[ncd][category]["month"][month_index].append(article["PMID"])

def mesh_term_present(article_mesh_terms, mesh_term):
        if isinstance(mesh_term, list):
            all_in = True
            for part in mesh_term:
                if part not in article_mesh_terms:
                    all_in = False
            
            return all_in
        else:
            return mesh_term in article_mesh_terms

CATEGORIES = [
    "NO KEYWORDS",
    "KEYWORDS",
    "SUBHEADINGS",
    "SITE PROPOSITION",
    "PROPOSITION"
]

INTERVALS = [
    "day",
    "week",
    "month"
]

DATA_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./data"))
SAVE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./save"))

file_path = f"{SAVE_DIR}/save_3_years.json"

with open(file_path, "r", encoding="utf-8") as file:
    data = json.load(file)

print(len(data))

ncds_mesh_terms = [mesh_term.lower() for ncd, mesh_term in NCDS_MESH_TERM.items()]
keywords_mesh_terms = lower_keywords(KEYWORDS_MESH_TERM)
keywords_subheading_mesh_terms = lower_keywords(KEYWORDS_MESH_SUBHEADING)
keywords_site_proposition_mesh_terms = lower_keywords(KEYWORDS_MESH_SITE_PROPOSITION)
keywords_proposition_mesh_terms = lower_keywords(KEYWORDS_MESH_PROPOSITION)

counts = {}

counts["ALL"] = {}

for category in CATEGORIES:
    counts["ALL"][category] = {}
    for interval in INTERVALS:
        counts["ALL"][category][interval] = {}

for ncd in ncds_mesh_terms:
    counts[ncd] = {}
    for category in CATEGORIES:
        counts[ncd][category] = {}
        for interval in INTERVALS:
            counts[ncd][category][interval] = {}


start_date = datetime(2022, 1, 1)
end_date = datetime(2025, 1, 30)
current_date = start_date

while(current_date < end_date):
    day_index, week_index, month_index = get_date_indices(current_date, start_date)

    for category in CATEGORIES:
        counts["ALL"][category]["day"][day_index] = []
        counts["ALL"][category]["week"][week_index] = []
        counts["ALL"][category]["month"][month_index] = []

    for ncd in ncds_mesh_terms:
        for category in CATEGORIES:
            counts[ncd][category]["day"][day_index] = []
            counts[ncd][category]["week"][week_index] = []
            counts[ncd][category]["month"][month_index] = []

    current_date += timedelta(days=1)

for article in data:

    article_mesh_terms = [mesh_term.lower() for mesh_term in article["MeshTerms"]]
    article_date = datetime(int(article["Date"]["Year"]), int(article["Date"]["Month"]), int(article["Date"]["Day"]))

    for ncd in ncds_mesh_terms:

        if mesh_term_present(article_mesh_terms, ncd):

            add_article(article, CATEGORIES[0], counts, ncd, article_date)
            add_article(article, CATEGORIES[0], counts, "ALL", article_date)

            added = False

            for keyword in keywords_mesh_terms:
                if added:
                    break

                if mesh_term_present(article_mesh_terms, keyword):

                    for category in CATEGORIES[1:]:
                        add_article(article, category, counts, ncd, article_date)
                        add_article(article, category, counts, "ALL", article_date)

                    added = True

            for keyword in keywords_subheading_mesh_terms:
                if added:
                    break

                if mesh_term_present(article_mesh_terms, keyword):

                    for category in CATEGORIES[2:]:
                        add_article(article, category, counts, ncd, article_date)
                        add_article(article, category, counts, "ALL", article_date)
                    
                    added = True

            for keyword in keywords_site_proposition_mesh_terms:
                if added:
                    break

                if mesh_term_present(article_mesh_terms, keyword):

                    for category in CATEGORIES[3:]:
                        add_article(article, category, counts, ncd, article_date)
                        add_article(article, category, counts, "ALL", article_date)

                    added = True

            for keyword in keywords_proposition_mesh_terms:
                if added:
                    break

                if mesh_term_present(article_mesh_terms, keyword):

                    for category in CATEGORIES[4:]:
                        add_article(article, category, counts, ncd, article_date)
                        add_article(article, category, counts, "ALL", article_date)

                    added = True

for ncd in ncds_mesh_terms:
    for category in CATEGORIES:
        for interval in INTERVALS:
            counts[ncd][category][interval] = [len(tmp) for _, tmp in counts[ncd][category][interval].items()]

            counts[ncd][category][interval] = {
                "min": min(counts[ncd][category][interval]),
                "max": max(counts[ncd][category][interval]),
                "mean": statistics.mean(counts[ncd][category][interval])
            }

for category in CATEGORIES:
        for interval in INTERVALS:
            counts["ALL"][category][interval] = [len(tmp) for key, tmp in counts["ALL"][category][interval].items()]

            counts["ALL"][category][interval] = {
                "min": min(counts["ALL"][category][interval]),
                "max": max(counts["ALL"][category][interval]),
                "mean": statistics.mean(counts["ALL"][category][interval])
            }

with open(f"{DATA_DIR}/articles_count_local_save.json", "w") as json_file:
    json.dump(counts, json_file, indent=4)