# -----------------------------
# Le script ne fait rien. Il met a disposition une fonction qui récupère des articles depuis PubMed selon 
# des paramètres puis récupère les données voulut des articles et les rend sous forme de liste.
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

import sys
import os

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))

from requests import get
from parsers.xmlParser import parseXmlFile
import json
from variables.pubmed import PUBMED_API_KEY
import xmltodict

TMP_DIR_NAME = "./tmp"
TMP_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), TMP_DIR_NAME))
TMP_FILENAME = "pubmedData.xml"

# term = "diabetes+type+1+OR+diabetes+type+2+OR+mental+health"
# term = '"Diabetes+Mellitus"[Mesh:noexp]'
# term = '"Diabetes+Mellitus%2C+Type+1"[Mesh:noexp]'
# term = '"Diabetes+Mellitus%2C+Type+2"[Mesh:noexp]'
# term = 'cancer'
# date_min = "2024/12/01"
# date_max = "2024/12/15"

def getPubmedData(term, date_min, date_max, nb_items = -1, debug = False, store = False, filename = ""):
    if store and (filename == "" or not filename.endswith(".json")):
        return False

    print(f"Term: {term}")
    print(f"Date min: {date_min}")
    print(f"Date max: {date_max}")

    url = f'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&api_key={PUBMED_API_KEY}&term={term}&retmode=json&mindate={date_min}&maxdate={date_max}&usehistory=y'

    while(True):
        try:  
            response = get(url)
            break
        except Exception as e:
            print(e)

    search_res = response.json()

    query_key = search_res["esearchresult"]["querykey"]
    webenv = search_res["esearchresult"]["webenv"]

    print(f"Query key: {query_key}")
    print(f"Web env: {webenv}")

    url = f"https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&query_key={query_key}&WebEnv={webenv}"

    while(True):
        try:
            response = get(url)
            break
        except Exception as e:
            print(e)

    with open(f"{TMP_DIR}/{TMP_FILENAME}", "w+", encoding="utf-8") as file:
        file.write(response.text)

    # obj = parseXmlFile(f"{TMP_DIR}/{TMP_FILENAME}")

    obj = xmltodict.parse(response.text)
    if "PubmedArticleSet" not in obj:
        return []
        
    obj = obj["PubmedArticleSet"]

    print()

    data_list = []

    for key in obj:
        if isinstance(obj[key], list):
            i = 0
            for entrie in obj[key]:
                if "MedlineCitation" in entrie:

                    # print("---------------------------------------------------------")

                    if "MeshHeadingList" in entrie["MedlineCitation"]:
                        data = {}
                        data["PMID"] = entrie["MedlineCitation"]["PMID"]["#text"]

                        data["Title"] = ""
                        if isinstance(entrie["MedlineCitation"]["Article"]["Journal"]["Title"], list):
                            for part in entrie["MedlineCitation"]["Article"]["Journal"]["Title"]:
                                if "#text" in part:
                                    data["Title"] += part["#text"]
                        elif "#text" in entrie["MedlineCitation"]["Article"]["Journal"]["Title"] and not isinstance(entrie["MedlineCitation"]["Article"]["Journal"]["Title"], str):
                            data["Title"] = entrie["MedlineCitation"]["Article"]["Journal"]["Title"]["#text"]
                        else:
                            data["Title"] = entrie["MedlineCitation"]["Article"]["Journal"]["Title"]
                        
                        data["ArticleTitle"] = ""
                        if isinstance(entrie["MedlineCitation"]["Article"]["ArticleTitle"], list):
                            for part in entrie["MedlineCitation"]["Article"]["ArticleTitle"]:
                                if "#text" in part:
                                    data["ArticleTitle"] += part["#text"]
                        elif "#text" in entrie["MedlineCitation"]["Article"]["ArticleTitle"] and not isinstance(entrie["MedlineCitation"]["Article"]["ArticleTitle"], str):
                            data["ArticleTitle"] = entrie["MedlineCitation"]["Article"]["ArticleTitle"]["#text"]
                        else:
                            data["ArticleTitle"] = entrie["MedlineCitation"]["Article"]["ArticleTitle"]
                        
                        data["Abstract"] = ""
                        if "Abstract" in entrie["MedlineCitation"]["Article"] :
                            if isinstance(entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"], list):
                                for part in entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"]:
                                    if "#text" in part:
                                        data["Abstract"] += part["#text"]
                            elif "#text" in entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"] and not isinstance(entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"], str):
                                data["Abstract"] = entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"]["#text"]
                            else:
                                data["Abstract"] = entrie["MedlineCitation"]["Article"]["Abstract"]["AbstractText"]
                        
                        data["Predictions"] = []

                        data["MeshTerms"] = []
                        if isinstance(entrie["MedlineCitation"]["MeshHeadingList"]["MeshHeading"], list):
                            for meshTerm in entrie["MedlineCitation"]["MeshHeadingList"]["MeshHeading"]:
                                data["MeshTerms"].append(meshTerm["DescriptorName"]["#text"])
                        else:
                            data["MeshTerms"].append(entrie["MedlineCitation"]["MeshHeadingList"]["MeshHeading"]["DescriptorName"]["#text"])

                        for date in entrie["PubmedData"]["History"]["PubMedPubDate"]:
                            if date["@PubStatus"] == "pubmed":
                                data["Date"] = {
                                    "Year": date["Year"],
                                    "Month": date["Month"],
                                    "Day": date["Day"]
                                }
                                break

                        if debug:
                            print(f"Index: {obj[key].index(entrie)}")
                            print(f"PMID: {data["PMID"]}")
                            print(f"Title: {data["Title"]}")
                            print(f"ArticleTitle: {data["ArticleTitle"]}")
                            print(f"Abstract: {data["Abstract"]}")
                            print(f"MeshTerms: {data["MeshTerms"]}")

                        data_list.append(data)

                        if nb_items > 0:
                            if i == nb_items:
                                break
                            else:
                                i+=1
    
    if store:
        with open(filename, "w") as json_file:
            json.dump(data_list, json_file, indent=4)

        return True
    else:
        return data_list