# Gemini

## API Key

To obtain an API Key...

## Limitation

### Quotas

When first using the api, i ran into a error: "google.api_core.exceptions.ResourceExhausted: 429 Resource has been exhausted (e.g. check quota).". I still haven't fully understand how this error arrives but i found an explication.

This error arrives when the number of requests exceeds the capacity allocated to process requests. Google has two quota framework: Pay-as-you-go and Provisioned Throughput.

For my case, the Pay-as-you-go framework is used. Because:

Pay-as-you-go -> Error message: "Resource exhausted, please try again later."
Provisioned Throughput -> Error message: "Too many requests. Exceeded the Provisioned Throughput."

#### Source

- Error code 429: https://cloud.google.com/vertex-ai/generative-ai/docs/error-code-429

### Pay-as-you-go

This framework uses dynamic shared quota (DSQ). 

Dynamic shared quota (DSQ) distributes available on-demand capacity among all queries being processed by Google Cloud services for specific models. This capability eliminates the need to set quota limits and to submit quota increase requests (QIRs).

DSQ processes requests from all customers to the same regional or multi-regional endpoints. Quotas are removed, and available capacity is distributed to each project.

To see wich google models support dynamic shared quota, see source "dynamic shared quota".

#### Retry strategy

To resolve the error code 429 problem they recomend to implement a retry strategy.

They recommend using an exponential backoff algorithm, that uses exponentially increasing waiting times between requests, up to a maximum backoff time.

#### Source

- dynamic shared quota: https://cloud.google.com/vertex-ai/generative-ai/docs/dsq
- retry strategy: https://cloud.google.com/storage/docs/retry-strategy#exponential-backoff

### Provisioned Throughput

Provisioned Throughput is a fixed-cost monthly subscription or weekly service that reserves throughput for supported generative AI models on Vertex AI. It is is the only way to ensure high availability for your application and to get predictable service levels for your production workloads.

To see wich google models are supported, see source "supported models".

To see purchase information, see source "Purchase information"

To see how to use this framework, see source "Usage"

#### Source

- Overview: https://cloud.google.com/vertex-ai/generative-ai/docs/provisioned-throughput
- Supported models: https://cloud.google.com/vertex-ai/generative-ai/docs/supported-models
- Purchase information: https://cloud.google.com/vertex-ai/generative-ai/docs/purchase-provisioned-throughput
- Usage: https://cloud.google.com/vertex-ai/generative-ai/docs/use-provisioned-throughput

### Gemini quotas

To see gemini quotas, go to: https://cloud.google.com/gemini/docs/quotas