# -----------------------------
# Le script calcule le prix moyen pour classifier les articles par jour, semaine, mois en utilisant le fichier des nombres de tokens (../../Tokenizer/data/token_count.json).
# Les prix des LLM sont récupérer d'internet et sont en $ par million de token. Les résultats sont stoquer dans ./data/llm_prices.json
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

from transformers import AutoTokenizer
import json
import sys
import os
import statistics
from datetime import datetime, timedelta

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../../")))

from testModel.utils import get_article_data
from variables.pubmed import NCDS_MESH_TERM, KEYWORDS_MESH_TERM, KEYWORDS_MESH_SUBHEADING, KEYWORDS_MESH_SITE_PROPOSITION, KEYWORDS_MESH_PROPOSITION

DATA_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "../../Tokenizer/data"))
RESULT_DATA_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./data"))

# Prices for 1M tokens
PRICES = {
    'Mistral Large': { 'input': 2, 'output': 6},
    'Mistral Small': { 'input': 0.1, 'output': 0.3},
    'GPT-4o': { 'input': 2.5, 'output': 10},
    'GPT-4o mini': { 'input': 0.15, 'output': 0.6},
    'DeepSeek-V3': { 'input': 0.27, 'output': 1.1},
    'Gemini 2.0 Flash': { 'input': 0.1, 'output': 0.4},
    'Gemini 2.0 Flash-Lite': { 'input': 0.075, 'output': 0.3},
    'Claude 3.7 Sonnet': { 'input': 3, 'output': 15},
    'Claude 3.5 Haiku': { 'input': 0.8, 'output': 4},
    'Claude 3 Opus': { 'input': 15, 'output': 75},
    'Command A': { 'input': 2.5, 'output': 10},
    'Command R+': { 'input': 2.5, 'output': 10},
}

INTERVALS = [
    "day",
    "week",
    "month"
]

CATEGORIES = [
    "NO KEYWORDS",
    "KEYWORDS",
    "SUBHEADINGS",
    "SITE PROPOSITION",
    "PROPOSITION"
]

TOKENIZERS = [
    #"openai-community/gpt-4",
    #"meta-llama/Llama-2-7b-hf",
    "bert-base-uncased",
    "roberta-base",
    "facebook/bart-large"
]

file_path = f"{DATA_DIR}/token_count.json"
with open(file_path, "r", encoding="utf-8") as file:
    data = json.load(file)

mean_data = {
    "ALL": {
        "input":0,
        "output":0
    }
}

for category in CATEGORIES:
    mean_data[category] = {}
    for interval in INTERVALS:
        mean_data[category][interval] = {
            "input": 0,
            "output": 0
        }


for tokenizer_name in TOKENIZERS:
    mean_data["ALL"]["input"] += data[tokenizer_name]["ALL"]["input"]
    mean_data["ALL"]["output"] += data[tokenizer_name]["ALL"]["output"]
    for category in CATEGORIES:
        for interval in INTERVALS:
            for i in ["input", "output"]:
                mean_data[category][interval][i] += data[tokenizer_name][category][interval][i]["mean"]

mean_data["ALL"]["input"] /= len(TOKENIZERS)
mean_data["ALL"]["output"] /= len(TOKENIZERS)
for category in CATEGORIES:
    for interval in INTERVALS:
        for i in ["input", "output"]:
            mean_data[category][interval][i] /= len(tokenizer_name)


res = {}

for llm in PRICES.keys():

    res[llm] = {
        "ALL": 0
    }

    for category in CATEGORIES:
        res[llm][category] = {}
        for interval in INTERVALS:
            res[llm][category][interval] = 0

for llm, prices in PRICES.items():
    res[llm]["ALL"] = prices["input"] / 1000000 * mean_data["ALL"]["input"] + prices["output"] / 1000000 * mean_data["ALL"]["output"]
    for category in CATEGORIES:
        for interval in INTERVALS:
            res[llm][category][interval] = prices["input"] / 1000000 * mean_data[category][interval]["input"] + prices["output"] / 1000000 * mean_data[category][interval]["output"]

with open(f"{RESULT_DATA_DIR}/llm_prices.json", "w") as json_file:
    json.dump(res, json_file, indent=4)