import json

# parseJsonFile reads a json file and parses it content
# :param filename: json file location and name 
# :return: parsed data from the json file
def parseJsonFile(filename):
    # Check if the file is a json file
    if not filename.lower().endswith('.json'):
        raise ValueError(f"The file {filename} is not a .json file")

    try:
        # Opens the specified file in read mode ('r') with UTF-8 encoding
        with open(filename, 'r', encoding='utf-8') as file:
            # Uses 'json.load()' to read and parse the content of the JSON file
            data = json.load(file)
        return data
    except json.JSONDecodeError:
        raise ValueError(f"The file ({filename}) content is not valid JSON")
    except FileNotFoundError:
        raise FileNotFoundError(f"The file {filename} was not found")
    except Exception as e:
        raise Exception(f"An error occurred while trying to parse the {filename} file: {e}")
   