# -----------------------------
# Le script transfer les fichier de testModel, models, parsers, variables et dateSources à un remote grâce à rclone copy
#
# Arguments:
# --remote : nom du remote où transferer les fichier (default: bamboo)
#
# Auteur : Pavlovich Ivan
# -----------------------------

import argparse
import os
import sys

REMOTES = {
    "bamboo": { "remote": "hpc-cluster-bamboo", "destination":"/home/users/p/pavlovii/NCD-Project"},
    "baobab": { "remote": "hpc-cluster-baobab", "destination":"/home/users/p/pavlovii/NCD-Project"},
    "yggdrasil": { "remote": "hpc-cluster-yggdrasil", "destination":"/home/users/p/pavlovii/NCD-Project"},
    "phenix": { "remote": "phenix-service", "destination": "/home/HES/ivan.pavlovic/NCD-Project"},
    "laptop": { "remote": "anthoine-laptop", "destination": "/home/guest/Documents/NCD-Project"},
    "tower": { "remote": "anthoine", "destination": "/home/guest/Documents/NCD-Project"}
}

parser = argparse.ArgumentParser(description="Sync project files to a remote.")

parser.add_argument(
    "--remote", 
    type=str, 
    default="bamboo",
    help="Specify the remote to use (default: bamboo)"
)

args = parser.parse_args()

selected_remote = REMOTES[args.remote]

if selected_remote is None:
    print(f"Error: No cluster selected. Choises: {[remote for remote in REMOTES.keys()]}")
    parser.print_help()
    exit(1)

PROJECT_PWD = os.path.abspath(os.path.join(os.path.dirname(__file__), "../"))

commands = [
    f"rclone copy {PROJECT_PWD}/models {selected_remote["remote"]}:{selected_remote["destination"]}/models -P",
    f"rclone copy {PROJECT_PWD}/testModel {selected_remote["remote"]}:{selected_remote["destination"]}/testModel -P",
    f"rclone copy {PROJECT_PWD}/parsers {selected_remote["remote"]}:{selected_remote["destination"]}/parsers -P",
    f"rclone copy {PROJECT_PWD}/variables {selected_remote["remote"]}:{selected_remote["destination"]}/variables -P",
    f"rclone copy {PROJECT_PWD}/dataSources {selected_remote["remote"]}:{selected_remote["destination"]}/dataSources -P"
]

for cmd in commands:
    print(f"-> Running: {cmd}")
    os.system(cmd)

print(f"Files successfully copied to {selected_remote["remote"]}:{selected_remote["destination"]}")