# -----------------------------
# Le script récupère les articles pour chaque maladies pour la classification et les stoque dans un fichier du même nom.abs
# Il récupère 100 articles dont 25 courts, 25 moyens, 25 longs et 25 très long pour chaque maladies.
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

import sys
import os
import json
import time
from datetime import datetime
from dateutil.relativedelta import relativedelta

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))

from dataSources.PubMed.pubmedApi import getPubmedData
from dataSources.PubMed.util import get_mesh_noexp_term, url_encode
from variables.pubmed import NCDS, NCDS_MESH_TERM
from variables.articles import LENGTH_CATEGORIES, LENGTH_CATEGORIES_TRESHOLDS
from testModel.utils import get_article_data, get_dataset_filename

DATASET_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./"))

date_min = "2024/01/01"
date_max = "2024/12/31"

ncds_mesh_noexp = get_mesh_noexp_term(NCDS_MESH_TERM)

stored_articles = []

for id_ncd, ncd in enumerate(NCDS):

    stored_length = {}
    for length_category in LENGTH_CATEGORIES:
        stored_length[length_category] = 0

    search_term = url_encode(ncds_mesh_noexp[id_ncd])

    data_store = []

    end_date = datetime(2024, 12, 31)

    while (len(data_store) < 100):

        start_date = end_date - relativedelta(years=1)

        data_list = getPubmedData(search_term, start_date.strftime("%Y/%m/%d"), end_date.strftime("%Y/%m/%d"))

        for data in data_list:
            if data["PMID"] in stored_articles:
                continue

            article_mesh_terms = [mesh_term.lower() for mesh_term in data["MeshTerms"]]

            for key, mesh_term in NCDS_MESH_TERM.items():
                if mesh_term.lower() in article_mesh_terms:
                    data["Predictions"].append(key)
            
            if len(data["Predictions"]) > 0:
                title, abstract = get_article_data(data)

                store = False
                breaked = False
                for id_treshold,  treshold in enumerate(LENGTH_CATEGORIES_TRESHOLDS):
                    if len(title+abstract) < treshold:
                        if stored_length[LENGTH_CATEGORIES[id_treshold]] < 25:
                            store = True
                            stored_length[LENGTH_CATEGORIES[id_treshold]] += 1
                        
                        breaked = True
                        break

                if not store and not breaked:
                    if stored_length[LENGTH_CATEGORIES[-1]] < 25:
                        store = True
                        stored_length[LENGTH_CATEGORIES[-1]] += 1

                if store:
                    data_store.append(data)
                    stored_articles.append(data["PMID"])
            
            if len(data_store) == 100:
                break

        end_date = start_date

    print(stored_length)

    filename = get_dataset_filename(ncd)

    with open(f"{DATASET_DIR}/{filename}.json", "w") as json_file:
        json.dump(data_store, json_file, indent=4)

    time.sleep(0.1)