# -----------------------------
# Le script regarde les statistiques du je de données test (nombre d'article par catégorie de longueur, longeur moyenne d'un article en caractères)
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

import sys
import os
import statistics

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from variables.pubmed import NCDS
from variables.articles import LENGTH_CATEGORIES, LENGTH_CATEGORIES_TRESHOLDS
from testModel.utils import get_dataset_filename, get_article_data
from parsers.jsonParser import parseJsonFile

DATASET_DIR_NAME = "./dataset"

DATASET_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), DATASET_DIR_NAME))

nb_articles = 0

nb_articles_categories = {}

for length_category in LENGTH_CATEGORIES:
    nb_articles_categories[length_category] = 0

lens = []

for ncd in NCDS:

    try:
        filename = get_dataset_filename(ncd)
        articles = parseJsonFile(f"{DATASET_DIR}/{filename}.json")
    except Exception as e:
        print(f"Error: {e}")

    for article in articles:
        nb_articles += 1

        title, abstract = get_article_data(article)

        text = title + abstract 

        lens.append(len(text))

        added = False
        for id, length_category_treshold in enumerate(LENGTH_CATEGORIES_TRESHOLDS):
            if len(text) < length_category_treshold:
                nb_articles_categories[LENGTH_CATEGORIES[id]] += 1
                added = True
                break

        if not added:
            nb_articles_categories[LENGTH_CATEGORIES[-1]] += 1

print(f"Nb articles: {nb_articles}")

for length_category in LENGTH_CATEGORIES:
    print(f"Nb articles ({length_category}): {nb_articles_categories[length_category]}")

print()

print(f"Longuest: {max(lens)}")
print(f"Shortest: {min(lens)}")
print(f"Mean: {statistics.mean(lens)}")
print(f"Median: {statistics.median(lens)}")