# -----------------------------
# Le script transfer les fichier de testModel, models, parsers, variables et dateSources à un remote grâce à rclone copy
#
# Arguments:
# --remote : nom du remote où transferer les fichier (default: bamboo)
#
# Auteur : Pavlovich Ivan
# -----------------------------

import subprocess
import sys
import os
import argparse

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from variables.models import MODELS, MODELS_CATEGORIES

TEST_SCRIPT_FILEPATH = "./test.py"

file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), TEST_SCRIPT_FILEPATH))

parser = argparse.ArgumentParser(description="Launch your_script.py with optional arguments.")
parser.add_argument(
    "--mode", choices=["ollama", "huggingface", "hosted", "fine-tuned"], help="Choose mode: ollama or huggingface", default=None
)
parser.add_argument("-r", "--remake", action="store_true", help="Run the test scripts with remake flag")

args = parser.parse_args()

if args.mode is None:
    models = MODELS.keys()
else:
    models = [model for model, params in MODELS.items() if params["category"] == args.mode]

for model in models:

    launch_args = ["-m", model]

    if args.remake:
        launch_args.append("-r")

    args_copy = launch_args

    while(True):
        try:
            subprocess.run(["python", file_path] + args_copy, check=True)
            break
        except subprocess.CalledProcessError as e:
            print("Script crashed with error:", e.stderr)
            if "-r" in args_copy:
                args_copy.remove("-r")
        except FileNotFoundError:
            print("Python executable or script not found.")
            break