import sys
import os

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from parsers.jsonParser import parseJsonFile
import time

from models.LLM.Cohere.cohereCommand import cohere_classify

LABELS = [
    "Noncommunicable Diseases",
    "Diabetes",
    "Cancer",
    "Chronic respiratory disease",
    "Cardiovascular diseases",
    "Mental Health",
    "Diabetes type 1",
    "Diabetes type 2"
]

def predict(article):
    title = article["Title"]
    articleTitle = article["ArticleTitle"]
    abstract = article["Abstract"]

    if not isinstance(articleTitle, str):
        articleTitle = ""

    if not isinstance(abstract, str):
        abstract = ""

    return cohere_classify(title + articleTitle + abstract, LABELS)

def get_wanted_predictions(article):
    wanted = {}
    for label in LABELS:
        wanted[label] = label in article["Predictions"]

    return wanted

def confusion_matrix(wanted, prediction):
    matrix = [[0, 0], [0, 0]]
    for key in wanted.keys():
        if wanted[key]:
            if prediction[key]:
                matrix[0][0] += 1
            else:
                matrix[1][0] += 1
        else:
            if prediction[key]:
                matrix[0][1] += 1
            else:
                matrix[1][1] += 1
    
    return matrix

def test_model():
    with open(f"./results/cohere/v1.txt", "w") as file:

        print("---------------------------------", file=file)
        print(f"MODEL: Cohere", file=file)
        print("---------------------------------", file=file)

        nb_entries = 0
        result_matrix = [[0, 0], [0, 0]]
        start = time.time()

        for label in LABELS:

            try:
                filename = label.replace(" ", "_").replace(",", "").lower()
                print(filename)
                articles = parseJsonFile(f"./dataset/{filename}.json")
                # nb_entries += len(articles)
            except Exception as e:
                print(f"Error: {e}")

            for article in articles:
                nb_entries += 1
                print("---------------------------------", file=file)

                wanted = get_wanted_predictions(article)

                print(f"PMID: {article["PMID"]}")
                print(f"PMID: {article["PMID"]}", file=file)
                pred = article["Predictions"]
                print(f"Predictions: {pred}", file=file)
                print(f"MeshTerm: {article["MeshTerms"]}", file=file)

                # try:
                results = predict(article)
                # except :
                #     end = time.time()
                #     print("Time: ", end-start)
                #     print("Nb entries: ", nb_entries)

                predictions = {}

                selected_labels = results["labels"]

                for label in LABELS:
                    predictions[label] = label in selected_labels

                print(f"Wanted: {wanted}", file=file)
                print(f"Predicted: {predictions}", file=file)
                print(f"Selected labels: {selected_labels}", file=file)

                matrix = confusion_matrix(wanted, predictions)
                print(f"Confusion matrix: {matrix}", file=file)
                
                for i in range(2):
                    for j in range(2):
                        result_matrix[i][j] += matrix[i][j]

                print("---------------------------------", file=file)
                time.sleep(6)

        end = time.time()

        print(f"Time to classify all articles: {end-start} seconds", file=file)
        print(f"Result confusion matrix: {result_matrix}", file=file)

        tpr = result_matrix[0][0] / (result_matrix[0][0] + result_matrix[1][0])
        tnr =  result_matrix[1][1] / (result_matrix[1][1] + result_matrix[0][1])
        precision = result_matrix[0][0] / (result_matrix[0][0] + result_matrix[0][1])

        print(f"True Positive Rate (TPR): {tpr}", file=file)
        print(f"True Negative Rate (TNR): {tnr}", file=file)
        print(f"Precision: {precision}", file=file)
        print()

test_model()