import sys
import os

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from parsers.jsonParser import parseJsonFile
import time
import statistics

from models.ZeroShotClassifier.HuggingFace.zero_shot_classification import create_classifier, classify, MODELS

TRESHOLD = 0.7

KEYWORDS = [
        "Diabetes",
        "Diabetes type 1",
        "Diabetes type 2"
        ]

LABELS = [
    "Noncommunicable Diseases",
    "Diabetes",
    "Cancer",
    "Chronic respiratory disease",
    "Cardiovascular diseases",
    "Mental Health",
    "Diabetes type 1",
    "Diabetes type 2"
]

def predict(classifier, article, file):
    pmid = article["PMID"]
    title = article["Title"]
    articleTitle = article["ArticleTitle"]
    abstract = article["Abstract"]
    pred = article["Predictions"]
    meshTerms = article["MeshTerms"]
    # print(f"PMID: {pmid}", file=file)
    # print(f"Title: {title}")
    # print(f"ArticleTitle: {articleTitle}")
    # print(f"Abstract: {abstract}")
    # print(f"Predictions: {pred}", file=file)
    # print(f"MeshTerm: {meshTerms}", file=file)

    if not isinstance(articleTitle, str):
        articleTitle = ""

    if not isinstance(abstract, str):
        abstract = ""

    return classify(classifier, title + " \n " + articleTitle + " \n " + abstract)

def predict_label(classifier, article, label):
    title = article["Title"]
    articleTitle = article["ArticleTitle"]
    abstract = article["Abstract"]

    if not isinstance(articleTitle, str):
        articleTitle = ""

    if not isinstance(abstract, str):
        abstract = ""

    return classify(classifier, title + " \n " + articleTitle + " \n " + abstract, lables=[label])

def get_wanted_predictions(article):
    wanted = {}
    for label in LABELS:
        wanted[label] = label in article["Predictions"]

    return wanted

def confusion_matrix(wanted, prediction):
    matrix = [[0, 0], [0, 0]]
    for key in wanted.keys():
        if wanted[key]:
            if prediction[key]:
                matrix[0][0] += 1
            else:
                matrix[1][0] += 1
        else:
            if prediction[key]:
                matrix[0][1] += 1
            else:
                matrix[1][1] += 1
    
    return matrix


def test_model(model):
    classifier = create_classifier(model)
    filename = model.replace(" ", "_").replace("-", "_").replace(".", "_").replace("/", "-")
    with open(f"./results/zero_shot/v1/{filename}.txt", "w") as file:

        print("---------------------------------", file=file)
        print(f"MODEL: {model}", file=file)
        print(f"TRESHOLD: {TRESHOLD}", file=file)
        print("---------------------------------", file=file)

        nb_entries = 0
        result_matrix = [[0, 0], [0, 0]]
        start = time.time()

        for label in LABELS:

            try:
                filename = label.replace(" ", "_").replace(",", "").lower()
                print(filename)
                articles = parseJsonFile(f"./dataset/{filename}.json")
                print(articles)
                nb_entries += len(articles)
            except Exception as e:
                print(f"Error: {e}")

            for article in articles:
                print("---------------------------------", file=file)

                wanted = get_wanted_predictions(article)

                print(f"PMID: {article["PMID"]}", file=file)
                pred = article["Predictions"]
                print(f"Predictions: {pred}", file=file)
                print(f"MeshTerm: {article["MeshTerms"]}", file=file)

                predictions = {}

                selected_labels = []

                for label in LABELS:
                    results = predict_label(classifier, article, label)

                    print(f"Labels: {results["labels"]}", file=file)
                    print(f"Scores: {results["scores"]}", file=file)

                    predictions[label] = results["scores"][0] > TRESHOLD

                    if results["scores"][0] > TRESHOLD:
                        selected_labels.append(label)

                print(f"Wanted: {wanted}", file=file)
                print(f"Predicted: {predictions}", file=file)
                print(f"Selected labels: {selected_labels}", file=file)

                matrix = confusion_matrix(wanted, predictions)
                print(f"Confusion matrix: {matrix}", file=file)
                
                for i in range(2):
                    for j in range(2):
                        result_matrix[i][j] += matrix[i][j]


                print("---------------------------------", file=file)

        end = time.time()

        print(f"Time to classify all articles: {end-start} seconds", file=file)
        print(f"Result confusion matrix: {result_matrix}", file=file)

        tpr = result_matrix[0][0] / (result_matrix[0][0] + result_matrix[1][0])
        tnr =  result_matrix[1][1] / (result_matrix[1][1] + result_matrix[0][1])
        precision = result_matrix[0][0] / (result_matrix[0][0] + result_matrix[0][1])

        print(f"True Positive Rate (TPR): {tpr}", file=file)
        print(f"True Negative Rate (TNR): {tnr}", file=file)
        print(f"Precision: {precision}", file=file)
        print()

for model in MODELS:
    test_model(model)