# -----------------------------
# Le script ne fait rien. Il met a disposition des fonctions utiles
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

import os
import sys
import json

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from testModel.metrics import get_confusion_matrix, get_label_confusion_matrix, get_tpr, get_tnr, get_precision, get_error_rate, get_f1_score, add_confusion_matrices
from variables.articles import LENGTH_CATEGORIES

def get_dataset_filename(name):
    return name.replace(" ", "_").replace(",", "").lower()

def get_article_data(article):
    title = article["ArticleTitle"]
    abstract = article["Abstract"]

    if not isinstance(title, str):
        title = ""

    if not isinstance(abstract, str):
        abstract = ""

    return title, abstract

def get_wanted_predictions(article, labels):
    wanted = {}
    for label in labels:
        wanted[label] = label in article["Predictions"]

    return wanted

def get_models_codensed_results_data(filename):
    if not os.path.exists(filename):
        with open(filename, "w") as file:
            json.dump([], file)

    with open(filename, "r") as file:
        try:
            data = json.load(file)
            if not isinstance(data, list):
                data = []
        except json.JSONDecodeError:
            data = []

    return data

def save_model_json(filepath, model, execution_time):
    filename = f"{filepath}/condensed_results.json"
    data = get_models_codensed_results_data(filename)

    exists = False
    for model_entry in data:
        if model_entry["model"] == model:
            exists = True
            break
    
    if not exists:
        data.append({
            "model": model,
            "execution_time": execution_time,
            "data": []
        })

    with open(filename, "w") as file:
        json.dump(data, file)

def save_model_results_json(filepath, model, new_entry):
    filename = f"{filepath}/condensed_results.json"
    data = get_models_codensed_results_data(filename)

    for model_entry in data:
        if model_entry["model"] == model:
            model_entry["data"].append(new_entry)

    with open(filename, "w") as file:
        json.dump(data, file)

def create_model_data_var(labels):
    data = {
        'treshold': None,
        'articles': [],
        'results': {}
    }

    data["results"]["ALL"] = create_results(labels)

    for length_category in LENGTH_CATEGORIES:
        data["results"][length_category] = create_results(labels)

    return data

def create_results(labels):
    res =  {
        'prediction time': 0,
        'gathering results time': 0,
        'confusion matrix': [[0, 0], [0, 0]],
        'labels': {},
        'TPR': 0,
        'TNR': 0,
        'Precision': 0,
        'Error rate': 0,
        'F1': 0
    }

    for label in labels:
        res["labels"][label] = [[0, 0], [0, 0]]

    return res

def store_results(results, wanted, predicted, predictionTime, gatheringTime):
    results["prediction time"] += predictionTime
    results["gathering results time"] += gatheringTime
    results["confusion matrix"] = add_confusion_matrices(results["confusion matrix"], get_confusion_matrix(wanted, predicted))

    for label, confusion_matrix in results["labels"].items():
        confusion_matrix = add_confusion_matrices(confusion_matrix, get_label_confusion_matrix(label, wanted, predicted))

    results["TPR"] = get_tpr(results["confusion matrix"])
    results["TNR"] = get_tnr(results["confusion matrix"])
    results["Precision"] = get_precision(results["confusion matrix"])
    results["Error rate"] = get_error_rate(results["confusion matrix"])
    results["F1"] = get_f1_score(results["confusion matrix"])

    return results
