from requests import get
from datetime import datetime, timedelta
import time
import threading
import os

KEYWORDS = [
    "Availability", "Affordability", "Essential medecins", "Care therapy", "Care health", "Health Expenditures", "Health care costs", "Market", "Special populations", "Child Health", "Womens Health", "Age", "Minority", "Primary Care", "Specialty Care", "Patient acceptance", "Patient centered care", "Prevention and control", "Mass screening", "Palliative care", "Quality", "Telemedicine", "Digital health", "Supplies", "Human Resources", "Enablers/barriers", "Gender equity", "Racial", "Equity", "Clinical", "Health promotion", "Health education", "Research & Innovation", "Therapeutic Development", "Technological Development", "Self-management", "Self-monitoring", "Dosing", "Injections", "Primary Care", "Secondary Care", "Integrated Care", "Treatment management", "Immunization", "Vaccination", "Prevention and control", "Mass screening", "Palliative care", "Adherence", "Control", "Rehabilitation services", "Clinical guidelines", "Health policy", "Healthcare policy", "National health policy", "Regional health policy", "Health legislation", "Policy evaluation", "Policy analysis", "Policy formulation", "Regulation", "Governance", "Global initiatives and organizations ", "Universal Health Care", "Expansion", "Health insurance", "Coverage", "Funding and investment", "Health planning", "Health reform", "Policy monitoring", "Public health campaign", "Policy lobbying", "Patient advocacy", "Justice", "Awareness campaign", "Education", "Corporate accountability", "Social determinants of health", "Empowerment", "Community", "Peer support", "Civil society", "Patient education", "Parent education", "Educational materials", "Community heatlh education", "Awareness ", "Community engagement", "Health literacy", "Medical education", "Training program", "Technology education", "Medical devices", "Information Dissemination", "Digital health", "Behavioral change", "Nutrition education", "Risk communication", "Sector integration"
]

TERMS = [
    '"Noncommunicable+Diseases"',       # NCDs (All)
    '"Diabetes+Mellitus"',              # Diabetes (type 1 or 2)
    '"Neoplasms"',                      # Cancer
    '"Respiratory+Tract+Diseases"',     # Chronic respiratory disease
    '"Cardiovascular+Diseases"',        # Cardiovascular diseases
    '"Mental+Health"',                  # Mental Health
    '"Diabetes+Mellitus%2C+Type+1"',    # Diabetes type 1
    '"Diabetes+Mellitus%2C+Type+2"'     # Diabetes type 2
]

INTERVALS = [
    #"day",
    #"week",
    "month"
]

RESULTS_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./results"))

def get_count_for_year(year, term, interval = "month"):
    current_date = datetime(year, 1, 1)
    counts = []

    while(current_date < datetime(year, 12, 31)):
        if interval == "day":
            next_date = current_date + timedelta(days=1)
        elif interval == "week":
            next_date = current_date + timedelta(weeks=1)
        elif interval == "month":
            next_date = (current_date.replace(day=28) + timedelta(days=4)).replace(day=1)

        url = f'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term={term}&retmode=json&mindate={current_date.strftime("%Y/%m/%d")}&maxdate={next_date.strftime("%Y/%m/%d")}&usehistory=y'
        response = get(url)
        search_res = response.json()
        counts.append(int(search_res["esearchresult"]["count"]))

        current_date = next_date
        time.sleep(1) # si plus de 3 requêtes par seconde sinon adresse IP bann (normalement)

    max_count = max(counts)
    min_count = min(counts)
    avg_count = sum(counts) / len(counts)

    return {"max": max_count, "min": min_count, "avg": avg_count}

data = {}

for term in TERMS:
    data[term] = {}

    ncd_mesh = term + "[Mesh]"
    print("TERM: ", ncd_mesh)

    keywords_term = ""

    for keyword in KEYWORDS:

        keyword_term = keyword.replace(" ", "+").replace("&", "%26").replace("/", "%2F")

        print("KEYWORD: ", keyword_term)

        keywords_term += keyword_term + "+OR+"

    search_term = ncd_mesh + "+AND+(+" + keywords_term + "+)"

    print("SEARCH: ", search_term)

    for interval in INTERVALS:
        print("INTERVAL: ", interval)
        counts = get_count_for_year(2024, search_term, interval)
        print(counts)
        data[term][interval] = counts

    # with open(f"{RESULTS_DIR}/{term}.txt", "w+") as file:
    #     print(data[term], file=file)

print("DATA: ", data)