import sys
import os
from requests import get
from datetime import datetime, timedelta
import time
import urllib.parse

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))

from variables.pubmed import *
from dataSources.PubMed.util import *

RESULTS_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./results"))

INTERVALS = [
    # "day",
    # "week",
    "month"
]

def get_count_for_year(year, term, interval = "month"):
    current_date = datetime(year, 1, 1)

    counts = []

    while(current_date < datetime(year, 12, 31)):
        if interval == "day":
            next_date = current_date + timedelta(days=1)
        elif interval == "week":
            next_date = current_date + timedelta(weeks=1)
        elif interval == "month":
            next_date = (current_date.replace(day=28) + timedelta(days=4)).replace(day=1)

        while (True):
            try:
                url = f'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&api_key={PUBMED_API_KEY}&term={term}&retmode=json&mindate={current_date.strftime("%Y/%m/%d")}&maxdate={next_date.strftime("%Y/%m/%d")}'
                print("Ok 1")
                response = get(url)
                print(response)
                print("Ok 2")
                search_res = response.json()
                print("Ok 3")
                counts.append(int(search_res["esearchresult"]["count"]))

                print("Ok 4")
                current_date = next_date
                break
            except Exception as e:
                print(e)
            
        time.sleep(0.2)

    max_count = max(counts)
    min_count = min(counts)
    avg_count = sum(counts) / len(counts)

    return {"max": max_count, "min": min_count, "avg": avg_count}

interval = "month"
tot = 0
for ncd in NCDS:
    search_term = url_encode(ncd)
    print(f"SEARCH_TERM: {search_term}")
    counts = get_count_for_year(2024, search_term, interval)

    tot += counts["avg"]

search_term = url_encode(" OR ".join(NCDS))
print(f"SEARCH_TERM: {search_term}")
counts = get_count_for_year(2024, search_term, interval)


print(tot)
print(counts["avg"])

exit(0)

result = {}

ncds_mesh = get_mesh_term(NCDS_MESH_TERM)
ncds_mesh_noexp = get_mesh_noexp_term(NCDS_MESH_TERM)

print(url_encode(" OR ".join(ncds_mesh)))
print(url_encode(" OR ".join(ncds_mesh_noexp)))

keywords_mesh = get_mesh_term(KEYWORDS_MESH_TERM)
keywords_mesh_noexp = get_mesh_noexp_term(KEYWORDS_MESH_TERM)

print(url_encode(" OR ".join(keywords_mesh)))
print(url_encode(" OR ".join(keywords_mesh_noexp)))

keywords_mesh_site_proposition = get_mesh_term(KEYWORDS_MESH_SITE_PROPOSITION)
keywords_mesh_site_proposition_noexp = get_mesh_noexp_term(KEYWORDS_MESH_SITE_PROPOSITION)

print(url_encode(" OR ".join(keywords_mesh_site_proposition)))
print(url_encode(" OR ".join(keywords_mesh_site_proposition_noexp)))

keywords_mesh_proposition = get_mesh_term(KEYWORDS_MESH_PROPOSITION)
keywords_mesh_proposition_noexp = get_mesh_noexp_term(KEYWORDS_MESH_PROPOSITION)

print(url_encode(" OR ".join(keywords_mesh_proposition)))
print(url_encode(" OR ".join(keywords_mesh_proposition_noexp)))

keywords_hubheading = [ f'"{subheading}"[Subheading]' for subheading in KEYWORDS_MESH_SUBHEADING]
keywords_hubheading_noexp = [ f'"{subheading}"[Subheading:noexp]' for subheading in KEYWORDS_MESH_SUBHEADING]

print(url_encode(" OR ".join(keywords_hubheading)))
print(url_encode(" OR ".join(keywords_hubheading_noexp)))

for interval in INTERVALS:
    result[interval] = {}

    for ncd in NCDS:
        result[interval][ncd] = {}

        search_term = url_encode(ncd)
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd]["WITHOUT_KEYWORDS"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd} AND ( " + " OR ".join(KEYWORDS) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd]["KEYWORDS"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd} AND ( " + " OR ".join(keywords_mesh) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd]["KEYWORDS_MESH"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd} AND ( " + " OR ".join(keywords_mesh_noexp) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd]["KEYWORDS_MESH_NOEXP"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd} AND ( " + " OR ".join(keywords_mesh + keywords_hubheading + keywords_mesh_site_proposition + keywords_mesh_proposition) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd]["KEYWORDS_MESH_ALL"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd} AND ( " + " OR ".join(keywords_mesh_noexp + keywords_hubheading_noexp + keywords_mesh_site_proposition_noexp + keywords_mesh_proposition_noexp) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd]["KEYWORDS_MESH_NOEXP_ALL"] = get_count_for_year(2024, search_term, interval)

    search_term = url_encode(" OR ".join(NCDS))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS"]["WITHOUT_KEYWORDS"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd} AND ( {" OR ".join(KEYWORDS)} ) )']
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS"]["KEYWORDS"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd} AND ( {" OR ".join(keywords_mesh)} ) )']
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS"]["KEYWORDS_MESH"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd} AND ( {" OR ".join(keywords_mesh_noexp)} ) )']
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS"]["KEYWORDS_MESH_NOEXP"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd} AND ( {" OR ".join(keywords_mesh + keywords_hubheading + keywords_mesh_site_proposition + keywords_mesh_proposition)} ) )']
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS"]["KEYWORDS_MESH_ALL"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd} AND ( {" OR ".join(keywords_mesh_noexp + keywords_hubheading_noexp + keywords_mesh_site_proposition_noexp + keywords_mesh_proposition_noexp)} ) )']
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS"]["KEYWORDS_MESH_NOEXP_ALL"] = get_count_for_year(2024, search_term, interval)

    # ------------------------------------

    for ncd_mesh in ncds_mesh:
        result[interval][ncd_mesh] = {}

        search_term = url_encode(ncd_mesh)
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh]["WITHOUT_KEYWORDS"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh} AND ( " + " OR ".join(KEYWORDS) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh]["KEYWORDS"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh} AND ( " + " OR ".join(keywords_mesh) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh]["KEYWORDS_MESH"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh} AND ( " + " OR ".join(keywords_mesh_noexp) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh]["KEYWORDS_MESH_NOEXP"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh} AND ( " + " OR ".join(keywords_mesh + keywords_hubheading + keywords_mesh_site_proposition + keywords_mesh_proposition) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh]["KEYWORDS_MESH_ALL"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh} AND ( " + " OR ".join(keywords_mesh_noexp + keywords_hubheading_noexp + keywords_mesh_site_proposition_noexp + keywords_mesh_proposition_noexp) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh]["KEYWORDS_MESH_NOEXP_ALL"] = get_count_for_year(2024, search_term, interval)

    search_term = url_encode(" OR ".join(ncds_mesh))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH"]["WITHOUT_KEYWORDS"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh} AND ( {" OR ".join(KEYWORDS)} ) )' for ncd_mesh in ncds_mesh]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH"]["KEYWORDS"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh} AND ( {" OR ".join(keywords_mesh)} ) )' for ncd_mesh in ncds_mesh]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH"]["KEYWORDS_MESH"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh} AND ( {" OR ".join(keywords_mesh_noexp)} ) )' for ncd_mesh in ncds_mesh]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH"]["KEYWORDS_MESH_NOEXP"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh} AND ( {" OR ".join(keywords_mesh + keywords_hubheading + keywords_mesh_site_proposition + keywords_mesh_proposition)} ) )' for ncd_mesh in ncds_mesh]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH"]["KEYWORDS_MESH_ALL"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh} AND ( {" OR ".join(keywords_mesh_noexp + keywords_hubheading_noexp + keywords_mesh_site_proposition_noexp + keywords_mesh_proposition_noexp)} ) )' for ncd_mesh in ncds_mesh]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH"]["KEYWORDS_MESH_NOEXP_ALL"] = get_count_for_year(2024, search_term, interval)

    # ----------------------------------------

    for ncd_mesh_noexp in ncds_mesh_noexp:
        result[interval][ncd_mesh_noexp] = {}

        search_term = url_encode(ncd_mesh_noexp)
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh_noexp]["WITHOUT_KEYWORDS"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh_noexp} AND ( " + " OR ".join(KEYWORDS) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh_noexp]["KEYWORDS"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh_noexp} AND ( " + " OR ".join(keywords_mesh) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh_noexp]["KEYWORDS_MESH"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh_noexp} AND ( " + " OR ".join(keywords_mesh_noexp) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh_noexp]["KEYWORDS_MESH_NOEXP"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh_noexp} AND ( " + " OR ".join(keywords_mesh + keywords_hubheading + keywords_mesh_site_proposition + keywords_mesh_proposition) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh_noexp]["KEYWORDS_MESH_ALL"] = get_count_for_year(2024, search_term, interval)

        search_term = url_encode(f"{ncd_mesh_noexp} AND ( " + " OR ".join(keywords_mesh_noexp + keywords_hubheading_noexp + keywords_mesh_site_proposition_noexp + keywords_mesh_proposition_noexp) + " )")
        print(f"SEARCH_TERM: {search_term}")
        result[interval][ncd_mesh_noexp]["KEYWORDS_MESH_NOEXP_ALL"] = get_count_for_year(2024, search_term, interval)

    search_term = url_encode(" OR ".join(ncds_mesh_noexp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH NOEXP"]["WITHOUT_KEYWORDS"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh_noexp} AND ( {" OR ".join(KEYWORDS)} ) )' for ncd_mesh_noexp in ncds_mesh_noexp]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH NOEXP"]["KEYWORDS"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh_noexp} AND ( {" OR ".join(keywords_mesh)} ) )' for ncd_mesh_noexp in ncds_mesh_noexp]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH NOEXP"]["KEYWORDS_MESH"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh_noexp} AND ( {" OR ".join(keywords_mesh_noexp)} ) )' for ncd_mesh_noexp in ncds_mesh_noexp]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH NOEXP"]["KEYWORDS_MESH_NOEXP"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh_noexp} AND ( {" OR ".join(keywords_mesh + keywords_hubheading + keywords_mesh_site_proposition + keywords_mesh_proposition)} ) )' for ncd_mesh_noexp in ncds_mesh_noexp]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH NOEXP"]["KEYWORDS_MESH_ALL"] = get_count_for_year(2024, search_term, interval)

    tmp = [f'( {ncd_mesh_noexp} AND ( {" OR ".join(keywords_mesh_noexp + keywords_hubheading_noexp + keywords_mesh_site_proposition_noexp + keywords_mesh_proposition_noexp)} ) )' for ncd_mesh_noexp in ncds_mesh_noexp]
    search_term = url_encode(" OR ".join(tmp))
    print(f"SEARCH_TERM: {search_term}")
    result[interval]["ALL NCDS MESH NOEXP"]["KEYWORDS_MESH_NOEXP_ALL"] = get_count_for_year(2024, search_term, interval)

print(result)

with open(f"{RESULTS_DIR}/results.json", "w+") as json_file:
    json.dump(result, json_file, indent=4)

print("END")