import json
import sys
import os
import statistics
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from requests import get
import time

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))

from variables.pubmed import PUBMED_API_KEY, NCDS_MESH_TERM
from dataSources.PubMed.util import get_mesh_noexp_term, url_encode

INTERVALS = [
    "day",
    "week",
    "month"
]

def get_count_for_year(year, term, interval = "month"):
    start_date = datetime(2021, 12, 31)

    counts = []

    while(start_date < datetime(2024, 12, 31)):
        if interval == "day":
            end_date = start_date + timedelta(days=1)
        elif interval == "week":
            end_date = start_date + timedelta(weeks=1)
        elif interval == "month":
            end_date = (start_date + timedelta(days=32)).replace(day=1)

        start_date += timedelta(days=1)

        print(f"Interval: {start_date.strftime("%Y/%m/%d")} - {end_date.strftime("%Y/%m/%d")}")
        url = f'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&api_key={PUBMED_API_KEY}&term={term}&retmode=json&mindate={start_date.strftime("%Y/%m/%d")}&maxdate={end_date.strftime("%Y/%m/%d")}'
        response = get(url)
        search_res = response.json()
        counts.append(int(search_res["esearchresult"]["count"]))

        start_date = end_date
        time.sleep(0.1) 
        
    max_count = max(counts)
    min_count = min(counts)
    avg_count = statistics.mean(counts)

    return {"max": max_count, "min": min_count, "avg": avg_count}

data = {}
data["ALL"] = {}

ncds_mesh_terms = get_mesh_noexp_term(NCDS_MESH_TERM)

for ncd in ncds_mesh_terms:
    data[ncd] = {}
    print("NCD: ", ncd)

    for interval in INTERVALS:
        print("INTERVAL: ", interval)
        counts = get_count_for_year(2024, url_encode(ncd), interval)
        print(counts)
        data[ncd][interval] = counts

print("ALL NCDS")
for interval in INTERVALS:
        print("INTERVAL: ", interval)
        counts = get_count_for_year(2024, url_encode(" OR ".join(ncds_mesh_terms)), interval)
        print(counts)
        data["ALL"][interval] = counts

DATA_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./data"))

with open(f"{DATA_DIR}/data_num_test.json", "w") as json_file:
    json.dump(data, json_file, indent=4)
