# https://medium.com/@lidores98/finetuning-huggingface-facebook-bart-model-2c758472e340
# Copied code: (need to understand and modify it for my usage)

import pandas as pd
import torch
from datasets import Dataset
import random
from transformers import BartTokenizerFast
from transformers import BartForSequenceClassification, Trainer, TrainingArguments, EvalPrediction
import numpy as np
from transformers import pipeline
import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))

from testModel.utils import get_dataset_filename, get_article_data
from parsers.jsonParser import parseJsonFile
from variables.pubmed import NCDS, NCDS_MESH_TERM

DATASET_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "../../testModel/dataset"))
TMP_DATA_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "../../dataSources/PubMed/tmp"))
DATA_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./data"))
LOGS_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./logs"))

# Get PMIDs for all the articles from my testing dataset
dataset_pmids = []

for ncd in NCDS:
    try:
        filename = get_dataset_filename(ncd)
        articles = parseJsonFile(f"{DATASET_DIR}/{filename}.json")
    except Exception as e:
        print(f"Error: {e}")

    dataset_pmids += [article["PMID"] for article in articles]

print(len(dataset_pmids))

# Remove article of the testing dataset from the training data
try:
  data = parseJsonFile(f"{TMP_DATA_DIR}/save_3_years.json")
except Exception as e:
  print(f"Error: {e}")

print(len(data))

articles = [article for article in data if article["PMID"] not in dataset_pmids]

# articles = articles[:1000]

print(len(articles))

dataset = []

for article in articles:
  title, abstract = get_article_data(article)
  text = title + abstract

  articles_mesh_terms = [mesh.lower() for mesh in article["MeshTerms"]]

  for ncd, mesh in NCDS_MESH_TERM.items():

    if mesh.lower() in articles_mesh_terms:
      dataset.append({
        "text": text,
        "label": ncd
      })

print(len(dataset))

label2id = {label: i for i, label in enumerate(NCDS)}

train_dataset = Dataset.from_dict({
    "text": [item["text"] for item in dataset],
    "label": [label2id[item["label"]] for item in dataset]
})

tokenizer = BartTokenizerFast.from_pretrained('facebook/bart-large-mnli')

def preprocess_function(examples):
    return tokenizer(examples["text"], truncation=True, padding="max_length")

tokenized_dataset = train_dataset.map(preprocess_function, batched=True)

tokenized_dataset = tokenized_dataset.remove_columns(["text"])
tokenized_dataset = tokenized_dataset.rename_column("label", "labels")
tokenized_dataset.set_format("torch")

print(len(tokenized_dataset))

training_args = TrainingArguments(
  output_dir = DATA_DIR,      # Output directory
  num_train_epochs = 32,             # Total number of training epochs
  per_device_train_batch_size = 12,  # Batch size per device during training
  warmup_steps = 500,                # Number of warmup steps for learning rate scheduler
  weight_decay = 0.01,               # Strength of weight decay
  logging_dir = LOGS_DIR,
  logging_steps=10,
  fp16=True,
  remove_unused_columns=False,
)

model = BartForSequenceClassification.from_pretrained("facebook/bart-large-mnli", num_labels = len(NCDS), ignore_mismatched_sizes=True)

device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
print(f"Using device: {device}")

if torch.cuda.device_count() > 1:
    print(f"Using {torch.cuda.device_count()} GPUs!")
    model = torch.nn.DataParallel(model)

model.to(device)

trainer = Trainer(
  model = model,                     # The instantiated model to be trained
  args = training_args,              # Training arguments, defined above
  train_dataset = tokenized_dataset,     # Training dataset
  tokenizer = tokenizer              # The tokenizer that was used
)

trainer.train()

# If the model is wrapped in DataParallel, access the underlying model using `model.module`
model_to_save = model.module if isinstance(model, torch.nn.DataParallel) else model

# Save the fine-tuned model
model_to_save.save_pretrained(DATA_DIR)
tokenizer.save_pretrained(DATA_DIR) 