from transformers import BartForSequenceClassification
from transformers import BartTokenizerFast
import os

SAVE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./save"))
MODEL_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./model"))

# Load the last checkpoint
from transformers.trainer_utils import get_last_checkpoint
last_checkpoint = get_last_checkpoint(SAVE_DIR)

if last_checkpoint:
    print(f"Loading model from checkpoint: {last_checkpoint}")
    model = BartForSequenceClassification.from_pretrained(last_checkpoint)
else:
    raise ValueError("No checkpoint found!")

# Save the model
model.save_pretrained(MODEL_DIR)
print(f"Model saved to {MODEL_DIR}")

tokenizer = BartTokenizerFast.from_pretrained('facebook/bart-large-mnli')
tokenizer.save_pretrained(MODEL_DIR)
