import sys
import os
import json

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../../")))

from variables.pubmed import NCDS
from transformers import pipeline
import torch

MODELS = [
    "deepseek-ai/DeepSeek-V3", #https://huggingface.co/deepseek-ai/DeepSeek-V3
    "meta-llama/Meta-Llama-3-8B",
    "microsoft/phi-2"
]

def create_generator(model = MODELS[0]):
    print(f" CUDA available: {torch.cuda.is_available()}")
    print(f"CUDA version: {torch.version.cuda}")
    print(f"GPUs number: {torch.cuda.device_count()}")
    device = 0 if torch.cuda.is_available() else -1
    return pipeline("text-generation", model=model, device=device)


def generate(generator, sequence, debug = False):
    results = generator(sequence)

    if debug:
        print(f"Sequence: {sequence}")
        print(f"Labels: {results['labels']}")
        print(f"Scores: {results['scores']}")

    return results

generator = create_generator(MODELS[2])

sequence = "Excessive daytime sleepiness and its predictors among type 2 diabetes mellitus patients at central ethiopia."
sequence += "Excessive daytime sleepiness is a common finding among type 2 diabetes mellitus patients. However there is scarce data that shows the magnitude of excessive daytime sleepiness, & its association with type 2 diabetes mellitus. Hence, the study aimed to assess the prevalence of excessive daytime sleepiness and its associated factors among type 2 diabetes mellitus patients at Wolkite University Specialized Hospital. A Hospital-based cross-sectional study was employed from January 15 to March 15, 2022, among 229 Type 2 diabetes mellitus patients. Data was collected by semi-structured questionnaires, then entered into the Epi data version 4.6 and exported to SPSS version 25.0 for analysis. Binary and multiple logistic regression analysis was used to assess factors associated with excessive daytime sleepiness and statistical significance was set at P-value\u2009<\u20090.05. The prevalence of Excessive daytime sleepiness among type 2 diabetes mellitus was 27.1%. Age (AOR: 1.08; 95%CI: 1.03, 1.12), frequent snoring (AOR: 2.9; 95%CI: 1.24, 6.80), comorbid hypertension (AOR: 2.64; 95%CI: 1.17, 5.96), obesity (AOR: 2.7; 95%CI: 1.03, 7.13), and poor glycemic control (AOR: 6.68; 95%CI: 1.83, 24.41) were independently associated with Excessive daytime sleepiness among type 2 diabetes mellitus patients. Excessive daytime sleepiness was reported in more than a quarter of type 2 diabetes mellitus patients. Age, frequent snoring, hypertension, obesity, and poor glycemic control were significantly associated with Excessive daytime sleepiness among type 2 diabetes mellitus patients. Therefore health care providers should assess not only for how well their patients' diabetes is controlled but also for excessive daytime sleepiness."

prompt = 'I need you to give me the labels that could be given to the text (keep in mind that u can put multiple labels and select only the labels that i give you):\n'
prompt += 'text: ' + sequence + '\n'
prompt += 'labels: ' + "[" + ", ".join(NCDS) + ']\n'
prompt += 'Give the response in json format: { "labels": [] }. Follow this writing to the letter and don t add text around it. Only put the labels that you select between [].'
prompt += 'Even if you have no labels give me a response like: { "labels": [] }. And also put labels between "".'

res = generate(generator, prompt)

print(res)
