import sys
import os

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from dataSources.PubMed.pubmedApi import getPubmedData
from transformers import AutoTokenizer, AutoModelForSequenceClassification
import time
import json

# term = '"Diabetes+Mellitus"[Mesh:noexp]'
# term = '"Diabetes+Mellitus%2C+Type+1"[Mesh:noexp]'
# term = '"Diabetes+Mellitus%2C+Type+2"[Mesh:noexp]'

date_min = "2024/10/01"
date_max = "2024/10/30"

pubmed_terms = ['"Diabetes+Mellitus"[Mesh:noexp]', '"Diabetes+Mellitus%2C+Type+1"[Mesh:noexp]', '"Diabetes+Mellitus%2C+Type+2"[Mesh:noexp]']
mesh_terms = ["Diabetes Mellitus", "Diabetes Mellitus, Type 1", "Diabetes Mellitus, Type 2"]
keywords = ["Diabetes", "Diabetes type 1", "Diabetes type 2"]

data_list = []

for term in pubmed_terms:
    data = getPubmedData(term, date_min, date_max, nb_items=100)
    data_list = data_list + data
    print("Sleep for 5 seconds")
    time.sleep(5)

with open("./data/training_data.json", "w") as json_file:
    json.dump(data_list, json_file, indent=4)

training_data = []

for data in data_list:
    labels = []
    for id, mesh in enumerate(mesh_terms):
        if mesh in data["MeshTerms"]:
            labels.append(id)

    # print(data["Title"])
    # print(data["ArticleTitle"])
    # print(data["Abstract"])

    if not isinstance(data["ArticleTitle"], str):
        data["ArticleTitle"] = ""

    if not isinstance(data["Abstract"], str):
        data["Abstract"] = ""
    
    training_data.append({
        "text": data["Title"] + " \n " + data["ArticleTitle"] + " \n " + data["Abstract"],
        "labels": labels
    })

with open("./data/training_data_paresed.json", "w") as json_file:
    json.dump(training_data, json_file, indent=4)

# # Load the BioBERT model (change model name for ClinicalBERT, PubMedBERT, etc.)
# model_name = "microsoft/BiomedNLP-BiomedBERT-base-uncased-abstract-fulltext"
# tokenizer = AutoTokenizer.from_pretrained(model_name)
# model = AutoModelForSequenceClassification.from_pretrained(model_name, num_labels=2)  # Adjust num_labels for your task

# # Example input text
# text = "The patient shows symptoms of hypertension and needs immediate attention."

# # Tokenize the text
# inputs = tokenizer(
#     text,
#     return_tensors="pt",  # Return PyTorch tensors
#     max_length=512,       # Max length for BioBERT (adjust based on task)
#     truncation=True,      # Truncate text longer than max_length
#     padding=True          # Pad text shorter than max_length
# )

# # Perform inference
# outputs = model(**inputs)

# # Get the predicted class (logits)
# logits = outputs.logits
# print("Logits: ", logits)

# predicted_class = logits.argmax(dim=-1).item()  # Class with highest score
# print(f"Predicted class: {predicted_class}")