import sys
import os

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '../')))

from models.HuggingFace.zero_shot_classification import create_classifier, classify
import models.LLM.Ollama.ollama_wrapper as ollama
from models.LLM.Gemini.gemini import gemini_start_chat, gemini_classify
from models.LLM.Cohere.cohere_wrapper import cohere_create_client, cohere_classify
from models.FineTuning.fine_tuning_wrapper import create_classifier_fine_tunning, classify_fine_tunning

MODELS_CATEGORIES = [
    "huggingface",
    "ollama",
    "hosted",
    "fine-tuned"
]

MODELS = {
    'facebook/bart-large-mnli': {'category': "huggingface", 'predict': classify, 'requiresDependency': True, 'dependency': create_classifier},                              # Params: 407M, Size: 1.6GB
    # 'MoritzLaurer/bge-m3-zeroshot-v2.0': {'category': "huggingface", 'predict': classify, 'requiresDependency': True, 'dependency': create_classifier}, 
    'MoritzLaurer/DeBERTa-v3-base-mnli-fever-anli': {'category': "huggingface", 'predict': classify, 'requiresDependency': True, 'dependency': create_classifier},          # Params: 184M, Size: 369MB
    'MoritzLaurer/deberta-v3-base-zeroshot-v1.1-all-33': {'category': "huggingface", 'predict': classify, 'requiresDependency': True, 'dependency': create_classifier},     # Params: 184M, Size: 369MB
    'MoritzLaurer/multilingual-MiniLMv2-L6-mnli-xnli': {'category': "huggingface", 'predict': classify, 'requiresDependency': True, 'dependency': create_classifier},       # Params: 107M, Size: 428MB
    # 'gritli/BioMed-left': {'category': "huggingface", 'predict': classify, 'requiresDependency': True, 'dependency': create_classifier},
    'llama3.2': {'category': "ollama", 'predict': ollama.classify, 'requiresDependency': False, 'dependency': None},                                                        # Params: 3B, Size: 2GB, 
    'mistral-small': {'category': "ollama", 'predict': ollama.classify, 'requiresDependency': False, 'dependency': None},                                                   # Params: 24B, Size: 14GB
    'deepseek-v2': {'category': "ollama", 'predict': ollama.classify, 'requiresDependency': False, 'dependency': None},                                                     # Params: 16B, Size: 8.9GB
    'gemini-hosted': {'category': "hosted", 'predict': gemini_classify, 'requiresDependency': True, 'dependency': gemini_start_chat},                                       # Params: , Input token limit: 1048576, Output token limit: 8192
    'cohere-hosted': {'category': "hosted", 'predict': cohere_classify, 'requiresDependency': True, 'dependency': cohere_create_client},                                    # Params: , Context: 128k, Output token limit: 4k
    'fine-tuned/bart-large-mnli': {'category': "fine-tuned", 'predict': classify_fine_tunning, 'requiresDependency': True, 'dependency': create_classifier_fine_tunning}
}